/*
 Copyright (C) 1997 Arve Kylling

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 1, or (at your option)
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 To obtain a copy of the GNU General Public License write to the
 Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../src_c/ascii.h"
#include "../src_c/numeric.h"
#include "../src_c/getopt.h"
#include "../src_f/f77-uscore.h"
#include "mie.h"

int mie_check(input_struct Input){
  int status=0, i;
  status =0;

  return status;
}

int mie(input_struct input, output_struct *output)
{
  int i, status = 0;
  
  complex ref;

  float pi, wl, temp;
  float *tmp_pmom;

  complex F77_FCN (refice) (float* wl, float* temp); 
  void F77_FCN (bhmie) ( float* xx, complex* crefin, int* numang,
			 complex* s1, complex* s2,  
			 float* qext, float* qsca, float* qback, float* gsca );
  void F77_FCN (miev0) ( float* xx, complex* crefin, int* perfct, float* mimcut, 
			 int* anyang, int* numang, float* xmu, int*  nmom, 
			 int* ipolzn, int* momdim, int* prnt, float* qext, 
			 float* qsca, float* gqsc, float*  pmom, complex* sforw, 
			 complex* sback, complex* s1, complex* s2, complex* tforw, 
			 complex* tback, float* spike );

  pi = 2.0*asin(1.0);

  temp = 300.; /* The temperature is only included for wavlengths > 167 um. 
		   Set an arbitrary value since were interested in the UV only. */

  output->wl.nlambda = (int)(input.wl.wl_end-input.wl.wl_start)/input.wl.wl_step;
  output->wl.lambda = (double *) calloc (output->wl.nlambda+1, sizeof(double));
  for (i=0;i<=output->wl.nlambda;i++) {
    output->wl.lambda[i] = input.wl.wl_start + input.wl.wl_step*i;
  }

  tmp_pmom  = (float *) calloc ((input.mie.nmom+1*4), sizeof(float));

  for (i=0;i<=output->wl.nlambda;i++) {
    wl = (output->wl.lambda[i])*1.e-03;  /* Convert from nm to um */
    ref = F77_FCN (refice) (&wl, &temp);
    output->mie.xx     = 2.*pi*input.r_mean/wl;
    output->mie.crefin = ref;
    /*printf ("xx %f  %f %f\n",output->mie.xx, input.r_mean, wl);*/

  switch (input.program) {
  case BHMIE:
    F77_FCN (bhmie) ( &output->mie.xx, &output->mie.crefin, &input.mie.numang,
		      input.mie.s1, input.mie.s2,  
		      &output->mie.qext, &output->mie.qsca, &output->mie.qback, 
		      &output->mie.gsca );
    break;
  case MIEV0:
    F77_FCN (miev0) ( &output->mie.xx, &output->mie.crefin, &input.mie.perfct, 
 		      &input.mie.mimcut, &input.mie.anyang, &input.mie.numang,  
 		      input.mie.xmu, &input.mie.nmom, &input.mie.ipolzn,  
 		      &input.mie.momdim, input.mie.prnt,  
 		      &output->mie.qext, &output->mie.qsca, &output->mie.gqsc,  
 		      tmp_pmom, &output->mie.sforw, &output->mie.sback,  
 		      input.mie.s1, input.mie.s2,  
 		      output->mie.tforw, output->mie.tback, &output->mie.spike ); 
    output->mie.gsca = output->mie.gqsc/output->mie.qsca;
    break;
  }

    printf("%f %e %e %f %e %f %e\n", 
	   output->wl.lambda[i], ref.re, ref.im, output->mie.qext, 
	   output->mie.qsca/output->mie.qext, 
	   output->mie.gsca, output->mie.spike);


  }

  return status;

}

